//
//  Treehousestudio.java
//  A simple Java applet
//

import java.awt.*;
import java.applet.*;
import java.util.*;

/**
* HOW TO ADD A MENU ITEM (cont.)
*
* If you haven't followed the instructions in the THSMenuItems class, go there before reading on.
*
* Now that you've made your own menu item with functionality and everything, it's time to place it in
* a menu to be displayed. To make a new menu category, like "File" and "Edit", instantiate a new THSMenu
* by telling the menubar object to add a Menu with the name you would like.  The addMenu() method then
* returns a THSMenu object which you can populate with menu items.  To add a menuitem, just use the
* makeMenuItem() method that you should have already updated, calling the static variable associated with
* your added menuitem.  Keep in mind that it does matter the order you add the menu items.  Use the spacer
* menu item to divide up the specific sections within the menu categories.  Make sure to follow the examples
* below, passing the right parameters into the makeMenuItem() method so that a proper menu item is produces
* by the factory method.
*
* Once you've done all of this, the menu item should appear in the menu bar the next time you compile and run
* the applet.
*/

public class Treehousestudio extends Applet {

    /***********************************************************************************
    ************************************************************************************
    ************************************************************************************
    **
    ** All of the following code can be modified to update the current Tree House Studio
    ** Applet so functionality can be added to the menu bar
    **
    ************************************************************************************
    ************************************************************************************
    ************************************************************************************/

    public void initMenus() {

        try {
            THSMenu fileMenu = menubar.addMenu("File");
            fileMenu.addMenuItem(THSMenuItems.makeMenuItem(THSMenuItems.SAVE, mainhb, gi, this));
            fileMenu.addMenuItem(THSMenuItems.makeMenuItem(THSMenuItems.OPEN, mainhb, gi, this));
    
            THSMenu editMenu = menubar.addMenu("Edit");
            editMenu.addMenuItem(THSMenuItems.makeMenuItem(THSMenuItems.UNDO, mainhb, gi, this));
            editMenu.addMenuItem(THSMenuItems.makeMenuItem(THSMenuItems.SPACER, mainhb, gi, this));
            editMenu.addMenuItem(THSMenuItems.makeMenuItem(THSMenuItems.CUT, mainhb, gi, this));
            editMenu.addMenuItem(THSMenuItems.makeMenuItem(THSMenuItems.COPY, mainhb, gi, this));
            editMenu.addMenuItem(THSMenuItems.makeMenuItem(THSMenuItems.PASTE, mainhb, gi, this));
            editMenu.addMenuItem(THSMenuItems.makeMenuItem(THSMenuItems.SPACER, mainhb, gi, this));
            editMenu.addMenuItem(THSMenuItems.makeMenuItem(THSMenuItems.SHOWCLIPBOARD, mainhb, gi, this));

            THSMenu testMenu = menubar.addMenu("Test");
            testMenu.addMenuItem(THSMenuItems.makeMenuItem(THSMenuItems.MAKEBRIGHTNESS5, mainhb, gi, this));
        } catch (RuntimeException re) {
            System.err.println("One of the menu items added wasn't properly instantiated either due to a typo or the fact that it hasn't been added to the THSMenuItems class yet.");
        }
        
    }
    
    /************************************************************************************
     ************************************************************************************
     ************************************************************************************
     **
     ** Everything else beyond this point is internal and shouldn't be changed unless
     ** you want to change an underlying aspect of the Applet.  Do use the rest of this code
     ** to understand how to add to the Applet.
     **
     ************************************************************************************
     ************************************************************************************
     ************************************************************************************/
    Image im=null;
    
    // Two sets of graphics
    Graphics gi;
    Graphics cg;
    
    int cbw, cbh;
    
    Vector panels = new Vector();
    Hashtable bh = new Hashtable();

    THSPanel mhitpanel;
    private int mbarh = 18;

    // These are the main components of the applet
    PaintCanvas mainhb;
    THSMenuBar menubar;
    THSIconPanel ib;
    ColorPalette colhb;

    Font menufont;
    FontMetrics fm;
    
    // These are the static variables that can be referenced by a call to reportMousePos
    static int mx = 0, my = 0;

    public void init() {
        setLayout(null);
    }

    public void appendhb(THSPanel a)
    {
        panels.addElement(a);
        bh.put(a.cs,a);
    }

    /**
     * This method returns the THSPanel that the mouse is within considering that it searches down to the most
     * specific child and if overlapping occurs, returns the first one to be added to the list
     * of hbs
     **/
    public THSPanel hitpanel(int x, int y)
    {
        int i;
        THSPanel a, res;
        for(i=0;i<panels.size();i++) {
            a = (THSPanel) panels.elementAt(i);
            if (a.insidep(x,y)) {
                return a;
            }
        }
        return null;
    }
    
    /** --------------------------------
     * These methods are all used to handle mouse actions
     *  -------------------------------- */
    
    public boolean mouseDown(Event ev, int x, int y) {
        mx = x;
        my = y;
        mhitpanel = hitpanel(x,y);

        if (mhitpanel != null) {
            mhitpanel.mouseDown(ev,x,y);
            redraw();
            displayGraphics();
        }
        
        return true;
    }

    public boolean mouseDrag(Event ev, int x, int y) {
        mx = x;
        my = y;

        if (mhitpanel != null) {
            
            mhitpanel.mouseDrag(ev,x,y);
            redraw();
            displayGraphics();
        }
        return true;
    }

    public boolean mouseUp(Event ev, int x, int y) {
        mx = x;
        my = y;
        if (mhitpanel!=null) {
            /*if (mhitpanel.getString() == "Menu") {
                redraw();
            }
                menubar.mouseUp(ev, x, y);
                displayGraphics();
                return true;
            }
            if (mhitpanel.getString() == "Paint Canvas") {
                ((PaintCanvas) mhitpanel).mouseUp(ev,x,y);
                return true;
            }
            if (mhitpanel.getString() == "Menu Bar") {
                return true;
            }*/

            mhitpanel.mouseUp(ev,x,y);
            redraw();
            displayGraphics();
        }

        mhitpanel = null;
        return true;
    }

    public boolean mouseMove(Event ev, int x, int y) {
        mhitpanel = hitpanel(x,y);
        if (mhitpanel != null) {
            if (mhitpanel.getString() == "Paint Canvas") {
                ((PaintCanvas) mhitpanel).mouseMove(ev,x,y);
                redraw();
                displayGraphics();
                return true;
            }
        }
        return false;
    }

    public static int[] reportMousePos() {
        int[] res = {mx, my};
        return res;
    }
    
    /**----------------------------------
     * This method handles all key actions
     * ---------------------------------- */

    public boolean keyDown(Event ev, int k) {
        if(mainhb.keyDown(ev,k))
        {
            redraw();
            displayGraphics();
        }
        return true;
    }

    
    /** --------------------------------
    * These methods are all used to handle the drawing and buffering of the drawn image
    *  -------------------------------- */
    
    public void redraw() {
        gi.setColor(Color.gray);
        gi.fillRect(0,0,cbw,cbh);
        gi.setColor(Color.black);
        gi.drawRect(0,0,cbw,cbh);

        for (int i = 0; i < panels.size(); i++) {
            THSPanel currPanel = (THSPanel) panels.elementAt(i);
            currPanel.draw(gi);
        }
    }

    public void displayGraphics() {
        cg.drawImage(im,0,0,this);
    }

    public void update(Graphics g) {
        paint(g);
    }

    public void paint(Graphics g) {
        Rectangle cb = this.bounds();
        System.out.println("got paint message");

        cbw = cb.width;
        cbh = cb.height;
        if (cbw>1) { // There needs to be at least a pixel's width of space
            if (im==null) {
                // This instantiates all of the items within the applet if the image has not been created
                // yet
                im = createImage(cbw,cbh);
                gi = im.getGraphics();
                cg = g.create();

                // Note that ordering of appending does matter with when each panel is drawn
                // the latter one's appended are drawn after the first ones.
                appendhb(mainhb = new PaintCanvas(80,30,cbw-100,cbh-80,this));
                appendhb(ib = new THSIconPanel(8,30,53,221,11,this));
                appendhb(colhb = new ColorPalette(8,ib.ch+40,53,53,11,this,true));
                appendhb(menubar = new THSMenuBar(gi, cbw, mbarh));
                checkBounds(ib.cw + mainhb.cw + 40, ib.ch + colhb.ch + 40);
                ib.setTarget(mainhb);
                colhb.setTarget(mainhb);
                menubar.setTarget(mainhb);
                
                ib.setFirstTool();
                initMenus();
            }
        } else return;

        redraw();
        displayGraphics();
    }

    public void checkBounds(int width, int height) {
        if (width > cbw) {
            cbw = width;
        }
        if (height > cbh) {
            cbh = height;
        }
    }
}
